from pubweb.cli.interactive import gather_upload_arguments, gather_download_arguments, gather_download_arguments_dataset, gather_login
from pubweb.auth import UsernameAndPasswordAuth
from pubweb.cli.models import UploadArguments, DownloadArguments
from pubweb.config import AuthConfig, save_config, load_config
from pubweb.file_utils import get_files_in_directory
from pubweb import PubWeb


def get_credentials():
    config = load_config()
    return config.username, config.password


def run_ingest(input_params: UploadArguments, interactive=False):
    pubweb = PubWeb(UsernameAndPasswordAuth(*get_credentials()))

    if interactive:
        projects = pubweb.project.list()
        processes = pubweb.process.list(process_type='INGEST')
        input_params = gather_upload_arguments(input_params, projects, processes)

    directory = input_params['data_directory']
    files = get_files_in_directory(directory)
    if len(files) == 0:
        raise RuntimeWarning("No files to upload, exiting")

    create_request = {
        'projectId': pubweb.project.get_project_id(input_params['project']),
        'processId': pubweb.process.get_process_id(input_params['process']),
        'name': input_params['name'],
        'description': input_params['description'],
        'files': files
    }

    create_resp = pubweb.dataset.create(create_request)
    pubweb.dataset.upload_files(dataset_id=create_resp['datasetId'],
                                project_id=create_request['projectId'],
                                directory=directory)


def run_download(input_params: DownloadArguments, interactive=False):
    pubweb = PubWeb(UsernameAndPasswordAuth(*get_credentials()))

    if interactive:
        projects = pubweb.project.list()
        input_params = gather_download_arguments(input_params, projects)

        input_params['project'] = pubweb.project.get_project_id(input_params['project'])
        datasets = pubweb.dataset.find_by_project(input_params['project'])
        input_params = gather_download_arguments_dataset(input_params, datasets)

    dataset_params = {
        'project': pubweb.project.get_project_id(input_params['project']),
        'dataset': input_params['dataset']
    }

    pubweb.dataset.download_files(project_id=dataset_params['project'],
                                  dataset_id=dataset_params['dataset'],
                                  download_location=input_params['data_directory'])


def run_configure():
    username, password = gather_login()
    auth_config = AuthConfig(username, password)
    save_config(auth_config)



import json
from typing import List

from gql import gql

from pubweb.clients import S3Client
from pubweb.clients.utils import filter_deleted, GET_FILE_ACCESS_TOKEN_QUERY
from pubweb.file_utils import upload_directory, download_directory
from pubweb.models.auth import Creds
from pubweb.models.dataset import CreateIngestDatasetInput, DatasetCreateResponse
from pubweb.services.base import BaseService
from pubweb.services.project import get_bucket


def _get_dataset_files(dataset_id: str, project_id: str, s3_client: S3Client) -> List[str]:
    manifest_path = f'datasets/{dataset_id}/web/manifest.json'
    file = s3_client.get_file(bucket=get_bucket(project_id), key=manifest_path)
    manifest = json.loads(file)
    return [file['file'] for file in manifest['files']]


class DatasetService(BaseService):
    def find_by_project(self, project_id):
        query = gql('''
          query DatasetsByProject(
            $project: ID!
            $sortDirection: ModelSortDirection
            $filter: ModelDatasetFilterInput
            $limit: Int
            $nextToken: String
          ) {
            datasetsByProject(
              project: $project
              sortDirection: $sortDirection
              filter: $filter
              limit: $limit
              nextToken: $nextToken
            ) {
              items {
                id
                status
                name
                desc
                sourceDatasets
                infoJson
                process
                createdAt
                updatedAt
                _deleted
              }
              nextToken
              startedAt
            }
          }
        ''')
        variables = {
            'project': project_id,
            'filter': {
                'status': {
                    'eq': 'COMPLETED'
                }
            }
        }
        resp = self._api_client.query(query, variables=variables)['datasetsByProject']
        return filter_deleted(resp['items'])

    def create(self, create_request: CreateIngestDatasetInput) -> DatasetCreateResponse:
        print(f"Creating dataset {create_request['name']}")
        query = gql('''
          mutation CreateIngestDataset($input: CreateIngestDatasetInput!) {
            createIngestDataset(input: $input) {
              datasetId
              dataPath
            }
          }
        ''')
        variables = {'input': create_request}
        data: DatasetCreateResponse = self._api_client.query(query, variables=variables)['createIngestDataset']
        print(f"Dataset ID: {data['datasetId']}")
        return data

    def upload_files(self, project_id: str, dataset_id: str, directory: str):
        if not dataset_id:
            raise RuntimeError('Dataset has not been created')
        token_request = {
            'projectId': project_id,
            'datasetId': dataset_id,
            'accessType': 'DATASET',
            'operation': 'UPLOAD'
        }
        variables = {'input': token_request}
        credentials_response = self._api_client.query(GET_FILE_ACCESS_TOKEN_QUERY,
                                                      variables=variables)
        credentials: Creds = credentials_response['getFileAccessToken']

        s3_client = S3Client(credentials)

        path = f'datasets/{dataset_id}/data'
        upload_directory(directory, s3_client, get_bucket(project_id), path)

    def download_files(self, project_id: str, dataset_id: str, download_location: str):
        token_request = {
            'projectId': project_id,
            'datasetId': dataset_id,
            'accessType': 'DATASET',
            'operation': 'DOWNLOAD'
        }
        variables = {'input': token_request}
        credentials_response = self._api_client.query(GET_FILE_ACCESS_TOKEN_QUERY,
                                                      variables=variables)
        credentials: Creds = credentials_response['getFileAccessToken']

        s3_client = S3Client(credentials)

        prefix = f'datasets/{dataset_id}'
        files = _get_dataset_files(dataset_id, project_id, s3_client)
        download_directory(download_location, s3_client, get_bucket(project_id), prefix, files)



from requests.auth import AuthBase

from pubweb.auth import AuthInfo


# TODO: Possible method of authenticating with API Keys
# since cognito auth doesn't work with federated login
class ClientAuth(AuthInfo):
    def get_request_auth(self) -> AuthBase:
        pass



from questionary import prompt


def prompt_wrapper(questions):
    answers = prompt(questions)
    # Prompt catches KeyboardInterrupt and sends back an empty dictionary
    # We want to catch this exception
    if len(answers) == 0:
        raise KeyboardInterrupt()
    return answers



from gql import gql

from pubweb.clients.utils import get_id_from_name, filter_deleted
from pubweb.services.base import BaseService


def get_bucket(project_id):
    return f'z-{project_id}'


class ProjectService(BaseService):
    def list(self):
        query = gql('''
          query ListProjects(
            $filter: ModelProjectFilterInput
            $limit: Int
            $nextToken: String
          ) {
            listProjects(filter: $filter, limit: $limit, nextToken: $nextToken) {
              items {
                id
                name
                desc
                _deleted
              }
            }
          }
        ''')

        resp = self._api_client.query(query)['listProjects']
        return filter_deleted(resp['items'])

    def get_project_id(self, name_or_id: str) -> str:
        return get_id_from_name(self.list(), name_or_id)



from pathlib import Path
from typing import List

from pubweb.clients import S3Client


def get_files_in_directory(directory) -> List[str]:
    path = Path(directory)
    path_posix = str(path.as_posix())

    paths = []

    for file_path in path.rglob("*"):
        if file_path.is_dir():
            continue
        str_file_path = str(file_path.as_posix())
        str_file_path = str_file_path.replace(f'{path_posix}/', "")
        paths.append(str_file_path)

    return paths


def get_directory_stats(directory):
    sizes = [f.stat().st_size for f in Path(directory).glob('**/*') if f.is_file()]
    return {
        'size': f'{sum(sizes) / float(1 << 30):,.3f} GB',
        'numberOfFiles': len(sizes)
    }


def upload_directory(directory: str, s3_client: S3Client, bucket: str, prefix: str):
    files = get_files_in_directory(directory)
    for file in files:
        key = f'{prefix}/{file}'
        local_path = Path(directory, file)

        s3_client.upload_file(local_path=local_path,
                              bucket=bucket,
                              key=key)


def download_directory(directory: str, s3_client: S3Client, bucket: str, prefix: str, files: List[str]):
    for file in files:
        key = f'{prefix}/{file}'
        local_path = Path(directory, file)
        local_path.parent.mkdir(parents=True, exist_ok=True)

        s3_client.download_file(local_path=local_path,
                                bucket=bucket,
                                key=key)



from typing import Dict

from gql import Client
from gql.transport.requests import RequestsHTTPTransport
from graphql import DocumentNode

from pubweb import config
from pubweb.auth.base import AuthInfo

HEADERS = {
    'Accept': 'application/json',
    'Content-Type': 'application/json'
}


def _build_gql_client(auth_info: AuthInfo, endpoint: str):
    transport = RequestsHTTPTransport(url=endpoint, headers=HEADERS, auth=auth_info.get_request_auth())
    return Client(transport=transport, fetch_schema_from_transport=True)


class ApiClient:
    def __init__(self, auth_info: AuthInfo):
        self._gql_client = _build_gql_client(auth_info, config.data_endpoint)

    def query(self, query: DocumentNode, variables=None) -> Dict:
        return self._gql_client.execute(query, variable_values=variables)



import boto3
from pycognito import AWSSRP
from requests.auth import AuthBase

from pubweb import config
from pubweb.auth import AuthInfo


class UsernameAndPasswordAuth(AuthInfo):
    def __init__(self, username, password):
        self.username = username
        self.password = password

    def get_request_auth(self) -> AuthBase:
        return self.RequestAuth(self._get_token()['AccessToken'])

    def _get_token(self):
        cognito = boto3.client('cognito-idp')
        aws = AWSSRP(username=self.username,
                     password=self.password,
                     pool_id=config.user_pool_id,
                     client_id=config.app_id,
                     client=cognito)
        resp = aws.authenticate_user()
        return resp['AuthenticationResult']

    class RequestAuth(AuthBase):
        def __init__(self, token):
            self.token = token

        def __call__(self, request):
            request.headers['Authorization'] = self.token
            return request



from typing import TypedDict, List


class DatasetCreateResponse(TypedDict):
    datasetId: str
    dataPath: str


class CreateIngestDatasetInput(TypedDict):
    name: str
    description: str
    processId: str
    projectId: str
    files: List[str]



from typing import List

from gql import gql


def get_id_from_name(items, name_or_id) -> str:
    matched = next((p for p in items if p['id'] == name_or_id), None)
    if matched:
        return matched['id']
    return next(p for p in items if p['name'] == name_or_id)['id']


def filter_deleted(items: List) -> List:
    return list(filter(lambda item: not item.get('_deleted', False), items))


GET_FILE_ACCESS_TOKEN_QUERY = gql('''
  query GetFileAccessToken($input: GetFileAccessTokenInput!) {
    getFileAccessToken(input: $input) {
      AccessKeyId
      Expiration
      SecretAccessKey
      SessionToken
    }
  }
''')



import configparser
import os
from pathlib import Path

PUBWEB_CONFIG_LOCATION = "~/.pubweb/config"


class AuthConfig:
    def __init__(self, username, password):
        self.username = username
        self.password = password


class DevelopmentConfig:
    user_pool_id = 'us-west-2_ViB3UFcvp'
    app_id = '2g2eg0g7tbjhbaa45diohmvqhs'
    data_endpoint = 'https://drdt2z4kljdbte5s4zx623kyk4.appsync-api.us-west-2.amazonaws.com/graphql'


class ProductionConfig:
    user_pool_id = 'us-west-2_LQnstneoZ'
    app_id = '7ic2n55r9h4fj0qej5q9ikr2o1'
    data_endpoint = 'https://22boctowkfbuzaidvbvwjxfnai.appsync-api.us-west-2.amazonaws.com/graphql'


if os.environ.get('ENV', '').upper() == 'DEV':
    config = DevelopmentConfig()
else:
    config = ProductionConfig()


def get_config_path() -> Path:
    config_location = os.environ.get('PW_CONFIG', PUBWEB_CONFIG_LOCATION)
    return Path(config_location).expanduser()


def save_config(auth_config: AuthConfig):
    ini_config = configparser.ConfigParser()
    ini_config['DEFAULT'] = auth_config.__dict__
    config_path = get_config_path()
    config_path.parent.mkdir(exist_ok=True)
    with config_path.open('w') as configfile:
        ini_config.write(configfile)


def load_config() -> AuthConfig:
    config_path = get_config_path()
    if not config_path.exists():
        raise RuntimeError('Please configure authentication by running pubweb-cli configure')
    ini_config = configparser.ConfigParser()
    ini_config.read(str(config_path.absolute()))
    auth_config = ini_config['DEFAULT']
    return AuthConfig(auth_config['username'], auth_config['password'])



from pubweb.auth import AuthInfo
from pubweb.clients import ApiClient
from pubweb.services import DatasetService, ProcessService, ProjectService


class PubWeb:
    def __init__(self, auth_info: AuthInfo):
        self._api_client = ApiClient(auth_info)
        self._dataset_service = DatasetService(self._api_client)
        self._project_service = ProjectService(self._api_client)
        self._process_service = ProcessService(self._api_client)

    @property
    def dataset(self):
        return self._dataset_service

    @property
    def project(self):
        return self._project_service

    @property
    def process(self):
        return self._process_service



import os

from pubweb.cli.interactive.prompt_wrapper import prompt_wrapper


def gather_login():
    answers = prompt_wrapper([
        {
            'type': 'input',
            'name': 'username',
            'message': 'Username',
            'default': os.environ.get('PW_USERNAME') or ''
        },
        {
            'type': 'password',
            'name': 'password',
            'message': 'Password'
        }
    ])
    return answers['username'], answers['password']



from gql import gql

from pubweb.clients.utils import get_id_from_name, filter_deleted
from pubweb.services.base import BaseService


class ProcessService(BaseService):
    def list(self, process_type=None):
        query = gql('''
          query ListProcesses(
            $filter: ModelProcessFilterInput
            $limit: Int
            $nextToken: String
          ) {
            listProcesses(filter: $filter, limit: $limit, nextToken: $nextToken) {
              items {
                id
                name
                desc
                _deleted
              }
            }
          }
        ''')
        item_filter = {}
        if process_type:
            item_filter['executor'] = {'eq': 'INGEST'}
        resp = self._api_client.query(query, variables={'filter': item_filter})['listProcesses']
        return filter_deleted(resp['items'])

    def get_process_id(self, name_or_id: str) -> str:
        return get_id_from_name(self.list(), name_or_id)



from setuptools import setup, find_packages
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

entry_points = {
    'console_scripts': [
        'pubweb-cli = pubweb.cli.cli:main'
    ]
}

install_requires = [
    "click>=8.0,<8.1",
    "boto3>=1.20,<1.30",
    "questionary==1.10.0",
    "gql[requests]==3.0.0",
    "requests==2.27.1",
    "pycognito==2022.1.0",
    "tqdm==4.62.3"
]

setup(
    name='pubweb',
    version='0.1.1',
    author='Fred Hutch',
    author_email='viz@fredhutch.org',
    description='CLI tool for interacting with the PubWeb platform',
    packages=find_packages(),
    install_requires=install_requires,
    url='https://github.com/FredHutch/PubWeb-client',
    entry_points=entry_points,
    # long_description=long_description
)



from pubweb.cli.interactive.auth_args import gather_login
from pubweb.cli.interactive.download_args import gather_download_arguments, gather_download_arguments_dataset
from pubweb.cli.interactive.upload_args import gather_upload_arguments



from pathlib import Path
from typing import List

from pubweb.cli.interactive.common_args import ask_project
from pubweb.cli.interactive.prompt_wrapper import prompt_wrapper
from pubweb.cli.models import DownloadArguments


def ask_dataset(datasets, input_value):
    if len(datasets) == 0:
        raise RuntimeWarning("No datasets available")

    dataset_prompt = {
        'type': 'list',
        'name': 'dataset',
        'message': 'What dataset would you like to download?',
        'choices': [f'{dataset["name"]} ({dataset["id"]})' for dataset in datasets]
    }
    answers = prompt_wrapper(dataset_prompt)
    choice = answers['dataset']
    return next(dataset for dataset in datasets if f'{dataset["name"]} ({dataset["id"]})' == choice)['id']


def ask_directory(input_value):
    directory_prompt = {
        'type': 'path',
        'name': 'directory',
        'only_directories': True,
        'message': 'Where would you like to download these files?',
        'default': input_value or str(Path.cwd())
    }

    answers = prompt_wrapper(directory_prompt)
    return answers['directory']


def gather_download_arguments(input_params: DownloadArguments, projects: List):
    input_params['data_directory'] = ask_directory(input_params.get('data_directory'))
    input_params['project'] = ask_project(projects, input_params.get('project'))
    return input_params


def gather_download_arguments_dataset(input_params: DownloadArguments, datasets: List):
    input_params['dataset'] = ask_dataset(datasets, input_params.get('dataset'))
    return input_params
